%Comsol model of cylinder in infinite ocean. The geometry consists of a 2d "dougnout". The thickness is equal to the radius of the cylinder. The boundaryconditions are given by numerical evaluation of an analytic expressen in the function etasolution.

clear all
flclear fem

% COMSOL version
clear vrsn
vrsn.name = 'COMSOL 3.2';
vrsn.ext = 'a';
vrsn.major = 0;
vrsn.build = 300;
vrsn.rcs = '$Name:  $';
vrsn.date = '$Date: 2005/12/20 19:02:30 $';
fem.version = vrsn;

% Constants. A is a scalingfactor, k is the wavenumber, tol decides how many terms to compute in the boundaryconditions and a is the radius of the cylinder.
fem.const = {'A','1', ...
  'k','2*pi', ...
  'tol','1e-8', ...
  'a','1'};
  
% Global expressions. Function for evaluation of the boundarycondition function (from the analytic expression).
%fem.globalexpr = {'etadiag','etasolutiondiag(sqrt(x.^2+y^2),atan2(y,x),k,a)'};


% Geometry. Make two circles and subtract the second from the first to make a "doughnut".
clear draw
g1=circ2(3);
g2=circ2(1);
g4=g1-g2;
draw.s.objs = {g4};
draw.s.name = {'CO1'};
draw.s.tags = {'g4'};
fem.draw = draw;
fem.geom = geomcsg(fem);

% Initialize mesh
fem.mesh=meshinit(fem);
%fem.mesh=meshinit(fem, ...
%                  'hmaxfact',0.8, ...
%                  'hgrad',1.2, ...
%                  'hcurve',0.25, ...
%                  'hcutoff',0.0003, ...
%                  'hpnt',1, ...
%                  'xscale',1, ...
%                  'yscale',1, ...
%                  'hgradedg',[3,1,4,1,6,1,7,1], ...
%                  'hcurveedg',[3,1,4,1,6,1,7,1], ...
%                  'hmaxedg',[3,0.1,4,0.1,6,0.1,7,0.1], ...
%                  'hcutoffedg',[3,1,4,1,6,1,7,1]);

% Refine mesh
%fem.mesh=meshrefine(fem, ...
%                    'mcase',0, ...
%                    'rmethod','regular');
%fem.mesh=meshrefine(fem, ...
%                    'mcase',0, ...
%                    'rmethod','regular');
%fem.mesh=meshrefine(fem, ...
%                    'mcase',0, ...
%                    'rmethod','regular');
%figure
%meshplot(fem)
% (Default values are not included)

% Application mode 1
clear appl
appl.mode.class = 'Helmholtz';
appl.assignsuffix = '_hzeq';
clear equ
equ.f = 0;
equ.a = 'k^2';
equ.c = -1;
appl.equ = equ;

clear bnd
bnd.ind = [1,1,2,2,1,2,2,1];
%figure
%The coefficients for the boundaryconditions are given as: h*u=r for Dirichlet and n*(c*nabla u)+q*u=g for Neumann. We have dirichlet at the outer boundary and neumann on the inner.
bnd.type = {'dir','neu'};
eta={'etasolutiondiag(sqrt(x.^2+y^2),atan2(y,x),k,a,1)'}
bnd.r ={'eta+eta*i','0'};
bnd.g = {0,0};
bnd.h = {1,0};
appl.bnd = bnd;
fem.appl{1} = appl;
fem.frame = {'ref'};
fem.border = 1;
fem.units = 'SI';

% Multiphysics
fem=multiphysics(fem);

% Extend mesh
fem.xmesh=meshextend(fem);

% Solve problem
fem.sol=femlin(fem, ...
               'solcomp',{'u'}, ...
               'outcomp',{'u'});


%Plot solution.
figure,hold on
postplot(fem, ...
         'tridata',{'abs(u)','cont','internal'}, ...
         'trimap','jet(1024)', ...
         'title','Absolute value of COOMSOL calculation')%, ...
         %'axis',[-3,3,-2,2.2,-1,1]);
%postplot(fem,'contdata','abs(u)')
%postplot(fem,'tridata','u','contdata','u.*x','triz','u','contz','u');
%postint(fem,'u')

%theta=-pi:0.001:pi;
%r=1.5*ones(1,length(theta));
%postvals=postinterp(fem,'u',[(r.*cos(theta));(r.*sin(theta))]);

%r=1.5;
%theta=-pi:0.001:pi;
%sol=etasolutionabs(r,theta,2*pi,1,1e-9);
%subplot(2,1,1)
%plot(theta,sol-abs(postvals),theta,zeros(1,length(theta)),'--');
%title('Absolute error')
%xlabel('\theta')
%ylabel('Error')
%axis([-pi pi -0.01 0.025])
%subplot(2,1,2)
%plot(theta,100*(abs(postvals)-sol)./sol,theta,zeros(1,length(theta)),'--');
%title('Relative error')
%xlabel('\theta')
%ylabel('Error')
%axis([-pi pi -5 0.025])
